@@ -471,10 +471,10 @@ deal_with_x_crashes (GdmDisplay *d)
 
 		    /* some promised variables */
 		    g_setenv ("XLOG", xlog, TRUE);
-		    g_setenv ("BINDIR", EXPANDED_BINDIR, TRUE);
-		    g_setenv ("SBINDIR", EXPANDED_SBINDIR, TRUE);
-		    g_setenv ("LIBEXECDIR", EXPANDED_LIBEXECDIR, TRUE);
-		    g_setenv ("SYSCONFDIR", EXPANDED_SYSCONFDIR, TRUE);
+		    g_setenv ("BINDIR", BINDIR, TRUE);
+		    g_setenv ("SBINDIR", SBINDIR, TRUE);
+		    g_setenv ("LIBEXECDIR", LIBEXECDIR, TRUE);
+		    g_setenv ("SYSCONFDIR", GDMCONFDIR, TRUE);
 
 		    /* To enable gettext stuff in the script */
 		    g_setenv ("TEXTDOMAIN", GETTEXT_PACKAGE, TRUE);
@@ -539,7 +539,7 @@ deal_with_x_crashes (GdmDisplay *d)
     /* if we have "open" we can talk to the user, not as user
      * friendly as the above script, but getting there */
     if ( ! just_abort &&
-	g_access (EXPANDED_LIBEXECDIR "/gdmopen", X_OK) == 0) {
+	g_access (LIBEXECDIR "/gdmopen", X_OK) == 0) {
 	    /* Shit if we knew what the program was to tell the user,
 	     * the above script would have been defined and we'd run
 	     * it for them */
@@ -679,6 +679,7 @@ gdm_cleanup_children (void)
     gint exitstatus = 0, status;
     GdmDisplay *d = NULL;
     gboolean crashed;
+    gboolean sysmenu;
 
     /* Pid and exit status of slave that died */
     pid = waitpid (-1, &exitstatus, WNOHANG);
@@ -766,7 +767,9 @@ gdm_cleanup_children (void)
     d->slavepid = 0;
     d->dispstat = DISPLAY_DEAD;
 
-    if ( ! gdm_get_value_bool (GDM_KEY_SYSTEM_MENU) &&
+    sysmenu = gdm_get_value_bool_per_display (d->name, GDM_KEY_SYSTEM_MENU);
+
+    if ( ! sysmenu &&
 	(status == DISPLAY_RESTARTGDM ||
 	 status == DISPLAY_REBOOT ||
 	 status == DISPLAY_SUSPEND ||
@@ -786,8 +789,8 @@ gdm_cleanup_children (void)
 
     if (status == DISPLAY_RUN_CHOOSER) {
 	    /* use the chooser on the next run (but only if allowed) */
-	    if (gdm_get_value_bool (GDM_KEY_SYSTEM_MENU) &&
-		gdm_get_value_bool (GDM_KEY_CHOOSER_BUTTON))
+	    if (sysmenu && 
+		gdm_get_value_bool_per_display (d->name, GDM_KEY_CHOOSER_BUTTON))
 		    d->use_chooser = TRUE;
 	    status = DISPLAY_REMANAGE;
 	    /* go around the display loop detection, these are short
@@ -2295,9 +2298,18 @@ gdm_handle_message (GdmConnection *conn, const char *msg, gpointer data)
 			send_slave_ack (d, NULL);
 		}
 	} else if (strcmp (msg, GDM_SOP_SUSPEND_MACHINE) == 0) {
+		GdmDisplay *d;
+		long slave_pid;
+		gboolean sysmenu;
+
+		if (sscanf (msg, GDM_SOP_CHOSEN_THEME " %ld", &slave_pid) != 1)
+			return;
+		d = gdm_display_lookup (slave_pid);
+
 		gdm_info (_("Master suspending..."));
-		if (gdm_get_value_string (GDM_KEY_SUSPEND) != NULL &&
-		    gdm_get_value_string (GDM_KEY_SYSTEM_MENU)) {
+
+		sysmenu = gdm_get_value_bool_per_display (d->name, GDM_KEY_SYSTEM_MENU);
+		if (sysmenu && gdm_get_value_string (GDM_KEY_SUSPEND) != NULL) {
 			suspend_machine ();
 		}
 	} else if (strncmp (msg, GDM_SOP_CHOSEN_THEME " ",
@@ -2821,7 +2833,7 @@ gdm_handle_user_message (GdmConnection *conn, const gchar *msg, gpointer data)
 	gdm_debug ("Handling user message: '%s'", msg);
 
 	if (gdm_connection_get_message_count (conn) > GDM_SUP_MAX_MESSAGES) {
-		gdm_debug ("Closing connection, %d messages reached");
+		gdm_debug ("Closing connection, %d messages reached", GDM_SUP_MAX_MESSAGES);
 		gdm_connection_write (conn, "ERROR 200 Too many messages\n");
 		gdm_connection_close (conn);
 		return;
@@ -3109,31 +3121,48 @@ gdm_handle_user_message (GdmConnection *conn, const gchar *msg, gpointer data)
 			gdm_connection_write (conn, "OK\n");
 	} else if (strncmp (msg, GDM_SUP_GET_CONFIG " ",
 		     strlen (GDM_SUP_GET_CONFIG " ")) == 0) {
-		const gchar *key = &msg[strlen (GDM_SUP_GET_CONFIG " ")];
-		gchar *retval;
+		const gchar *parms = &msg[strlen (GDM_SUP_GET_CONFIG " ")];
+		gchar **splitstr = g_strsplit (parms, " ", 2);
+		gchar *retval = NULL;
 		static gboolean done_prefetch = FALSE;
 
 		/*
-		 * If key is for prefetch do it only once
+		 * It is not meaningful to manage this in a per-display 
+		 * fashion since the prefetch program is only run once the
+		 * for the first display that requests the key.  So process
+		 * this first and return "Unsupported key" for requests
+		 * after the first request.
 		 */
-		if (strcmp(key, GDM_KEY_PRE_FETCH_PROGRAM) == 0) {
+		if (strcmp (splitstr[0], GDM_KEY_PRE_FETCH_PROGRAM) == 0) {
 			if (done_prefetch) {
 				gdm_connection_printf (conn, "OK \n");
-				return;
 			} else {
+                		gdm_connection_printf (conn, "ERROR 50 Unsupported key <%s>\n", splitstr[0]);
 				done_prefetch = TRUE;
 			}
+			return;
 		}
 
-                gdm_config_to_string ((gchar *)key, &retval);
-		if (retval != NULL) {
-			gdm_connection_printf (conn, "OK %s\n", retval);
-			g_free (retval);
-		} else {
-			if (gdm_is_valid_key ((gchar *)key))
-				gdm_connection_printf (conn, "OK \n");
-			else
-                		gdm_connection_printf (conn, "ERROR 50 Unsupported key <%s>\n", key);
+		if (splitstr[0] != NULL) {
+                       /*
+                        * Note passing in the display is backwards compatible 
+                        * since if it is NULL, it won't try to load the display
+			* value at all.
+                        */
+			gdm_config_to_string (splitstr[0], splitstr[1], &retval);
+
+	   		if (retval != NULL) {
+   				gdm_connection_printf (conn, "OK %s\n", retval);
+   				g_free (retval);
+			} else {
+				if (gdm_is_valid_key ((gchar *)splitstr[0]))
+					gdm_connection_printf (conn, "OK \n");
+				else
+       		         		gdm_connection_printf (conn,
+						"ERROR 50 Unsupported key <%s>\n",
+						splitstr[0]);
+			}
+			g_strfreev (splitstr);
 		}
 	} else if (strcmp (msg, GDM_SUP_GET_CONFIG_FILE) == 0) {
 		GString *msg;
@@ -3144,22 +3173,29 @@ gdm_handle_user_message (GdmConnection *conn, const gchar *msg, gpointer data)
 		g_string_free (msg, TRUE);
 	} else if (strcmp (msg, GDM_SUP_GET_CUSTOM_CONFIG_FILE) == 0) {
 		GString *msg;
+		gchar *ret;
 
 		msg = g_string_new ("OK");
 		g_string_append (msg, "\n");
-		gdm_connection_printf (conn, "OK %s\n", gdm_get_custom_config_file ());
+		ret = gdm_get_custom_config_file ();
+		if (ret)
+			gdm_connection_printf (conn, "OK %s\n", ret);
+		else
+			gdm_connection_write (conn,
+					      "ERROR 1 File not found\n");
 		g_string_free (msg, TRUE);
 	} else if (strcmp (msg, GDM_SUP_QUERY_LOGOUT_ACTION) == 0) {
-		const gchar *sep = " ";
-		GdmDisplay *disp;
 		GdmLogoutAction logout_action;
+		GdmDisplay *disp;
 		GString *msg;
-		gboolean sysmenu = gdm_get_value_bool (GDM_KEY_SYSTEM_MENU);
+		const gchar *sep = " ";
+		gboolean sysmenu;
 
 		disp = gdm_connection_get_display (conn);
+		sysmenu = gdm_get_value_bool_per_display (disp->name, GDM_KEY_SYSTEM_MENU);
 
 		/* Only allow locally authenticated connections */
-		if ( ! GDM_CONN_AUTHENTICATED(conn) ||
+		if ( ! GDM_CONN_AUTHENTICATED (conn) ||
 		     disp == NULL) {
 			gdm_info (_("%s request denied: "
 				    "Not authenticated"), "QUERY_LOGOUT_ACTION");
@@ -3204,9 +3240,10 @@ gdm_handle_user_message (GdmConnection *conn, const gchar *msg, gpointer data)
 			&msg[strlen (GDM_SUP_SET_LOGOUT_ACTION " ")];
 		GdmDisplay *disp;
 		gboolean was_ok = FALSE;
-		gboolean sysmenu = gdm_get_value_bool (GDM_KEY_SYSTEM_MENU);
+		gboolean sysmenu;
 
 		disp = gdm_connection_get_display (conn);
+		sysmenu = gdm_get_value_bool_per_display (disp->name, GDM_KEY_SYSTEM_MENU);
 
 		/* Only allow locally authenticated connections */
 		if ( ! GDM_CONN_AUTHENTICATED(conn) ||
@@ -3256,9 +3293,10 @@ gdm_handle_user_message (GdmConnection *conn, const gchar *msg, gpointer data)
 			&msg[strlen (GDM_SUP_SET_SAFE_LOGOUT_ACTION " ")];
 		GdmDisplay *disp;
 		gboolean was_ok = FALSE;
-		gboolean sysmenu = gdm_get_value_bool (GDM_KEY_SYSTEM_MENU);
+		gboolean sysmenu;
 
 		disp = gdm_connection_get_display (conn);
+		sysmenu = gdm_get_value_bool_per_display (disp->name, GDM_KEY_SYSTEM_MENU);
 
 		/* Only allow locally authenticated connections */
 		if ( ! GDM_CONN_AUTHENTICATED(conn) ||
